"""
.. currentmodule:: flytekitplugins.envd

This plugin enables seamless integration between Flyte and envd.

.. autosummary::
   :template: custom.rst
   :toctree: generated/

   EnvdImageSpecBuilder
"""

from .image_builder import EnvdImageSpecBuilder
